
# (C) Copyright 1996 University of Toronto
# Permission to use this software for non-commerical purposes is hereby
# granted.  This software is provided "as is" without express or implied
# warranty.  This software may not be transferred or sublicensed.
# Any enhancements made by the recipient of this software shall be
# shared with the original copyright holders.

# Acknowledgements
# This software was developed as part of the project entitled "Models
# for Supporting the Analysis and Redesign of Business Processes" funded
# by the Information Technology Research Centre of Ontario.
# Principal investigators for the project are John Mylopoulos, Ray
# Reiter, Hector Levesque, Eric Yu, and Yves Lesperance.
# The i* modelling framework (including the Strategic Dependency and
# Strategic Rationale models) was developed by Eric Yu in his doctoral
# dissertation, Department of Computer Science, University of Toronto,
# 1995.
# This software was originally written by Chen Wang as part of his
# Master degree requirement.
# Substantial modifications and enhancements were made by Godfrey Cheng
# and Seyil Yoon, who worked as summer students on this project.
# Further modifications were made by Mike Bissener
# The "repository" part of the code was originally written by Brian Kramer and
# subsequently modified by Chen Wang, Seyil Yoon, and Godfrey Cheng.


##########################################################################
# Call NFR Assistant Routines
##########################################################################
## not needed by this version of OME
proc select_softgoal {canvas} {
    global Display_Node_Text Display_Node_Tag
    
    set id [$canvas find withtag current]
    
    $canvas bind node <Button-1> "no_op"
    display_message $canvas ""
    set tag $Display_Node_Tag($canvas,$id)
    set softgoal [lindex $Display_Node_Text($canvas,$tag) 1]

    eval exec /h/i/wings/pub/nfra ./NFR_main.tcl $softgoal
}

proc call_NFR {canvas} {

    display_message $canvas "Softgoal"
    
    $canvas bind node <Button-1> "select_softgoal $canvas"
}

##########################################################################
# Handling Text Window
##########################################################################

proc create_tabbed_frame {parent {tabs 0}} {
    global text_no Display_Font
    
    set fn $Display_Font(Text)

    if {[info exists text_no]} {
        incr text_no
    } else {
        set text_no 0
    } 

    set f [frame $parent.f$text_no]

    pack [label $f.space -text "" -width [expr $tabs*5] -font $fn] \
        -side left -fill x

    return $f
}

proc validate_label {canvas label} {
    global Exist_Object Object_List
    
    foreach type $Object_List {
        if {[info exists Exist_Object($canvas,$type)]} {
            foreach obj $Exist_Object($canvas,$type) {
                if {$obj == $label} {
                    return 0
                }    
            }
        }
    }    
    return 1
}

proc validate_context {canvas context} {
    global Exist_Object Valid_Context_List
        
    foreach type $Valid_Context_List {
        if {[info exists Exist_Object($canvas,$type)]} {
            foreach obj $Exist_Object($canvas,$type) {
                if {$obj == $context} {
                    return 1
                }    
            }
        }
    }    
    return 0
}

proc action {cmd canvas {type ""}} {
    global Edit_Label Edit_Context User_Input dlgButton
    
    if {$User_Input(label) != ""} {
        
        ##### trim extra spaces and replace empty spaces with underscores #####
        set User_Input(label) [string trim $User_Input(label) " "]
        regsub -all " " $User_Input(label) "_" User_Input(label)
        
        if {[validate_label $canvas $User_Input(label)]} {
            set Edit_Label($canvas) $User_Input(label)
        } else {
            dialogBox .dlg "Duplicate names" "Object $User_Input(label) is \
                already exist" 0 OK
            return    
        }

        if {$User_Input(context) != "none"} {
            if {[validate_context $canvas $User_Input(context)]} {
                set Edit_Context($canvas) $User_Input(context)
            } else {
                dialogBox .dlg "Invalid context name" "Cannot WRT undefined object \
                    $User_Input(context)" 0 OK
                return    
            }
        } else {
            set Edit_Context($canvas) none
        }    
        eval $cmd
    }
}    

set User_Input(label) ""
set User_Input(context) none

proc get_context {context} {
    global User_Input

    if {$context == "none"} {
        set User_Input(context) none
    } else {
        set User_Input(context) [lindex $context 1]
    }
}

proc list_obj {canvas contf} {
    global Exist_Object Valid_Context_List User_Input
    
    set win [toplevel .listbox_win]
    wm title $win "Existing objects"
       
    listbox $win.listbox -relief raised -borderwidth 2 \
        -yscrollcommand "$win.scroll set"    
    pack [label $win.label -text "Please select an WRT object:"] \
        -padx 2m -pady 1m
    pack $win.listbox -side left -fill x -expand 1 -padx 2m -pady 1m
    scrollbar $win.scroll -command "$win.listbox yview"
    pack $win.scroll -side right -fill y
    
    $win.listbox insert end "none"
    foreach type $Valid_Context_List {
        if {[info exists Exist_Object($canvas,$type)]} {
            foreach obj $Exist_Object($canvas,$type) {
                $win.listbox insert end "$type $obj"
            }
        }
    }

    bind $win.listbox <Double-Button-1> {
        get_context [%W get [%W nearest %y]]
    }    
        
    set oldFocus [focus]
    catch {
        grab $win
    }
    focus $win
    
    tkwait variable User_Input(context)
    destroy $win
    focus $oldFocus
}

proc setNull {canvas} {
    global Edit_Label
    set Edit_Label($canvas) ""
}

proc create_text_edit {canvas type id {text .} {cmd ""} {internal 0}} {
    global Edit_Label Display_Font User_Input
    
    set fn $Display_Font(Text)
    
    set topf [create_tabbed_frame $text 0]
    set entry [entry $topf.entry -textvariable User_Input(label) \
             -relief sunken -width 20 -font $fn]
             
    pack [label $topf.label -text "TELL CLASS " -font $fn] $entry -side left
    pack $topf -side top -fill x -padx 2m

    bind $entry <Return> "action {$cmd} $canvas $type"
    
    if {$internal == 1} {
        set contf [create_tabbed_frame $text 1]
        pack [label $contf.label -text "With respect to" -font $fn] -side left
        pack [entry $contf.entry -width 20 -relief sunken \
            -textvariable User_Input(context)] -side left
        pack [button $contf.wrt -text more -font $fn \
            -command "list_obj $canvas $contf"] -side left
        pack $contf -side top -fill x -padx 2m
        bind $contf.entry <Return> "action {$cmd} $canvas $type"
    }
    
    set typef [create_tabbed_frame $text 1]
    pack [label $typef.label -text "IN ${type}Class" -font $fn] -side left
    pack $typef -side top -fill x

    set labelf [create_tabbed_frame $text 1]
    pack [label $labelf.label -text "WITH" -font $fn] -side left
    pack $labelf -side top -fill x
    
    set af [create_tabbed_frame $text 2]
    pack [label $af.label -text "attribute" -font $fn] -side left
    pack $af -fill x -side top

    set idf [create_tabbed_frame $text 3]
    pack [label $idf.id -text "Identifier : $id" -font $fn] -side left
    pack $idf -fill x -side top
    
    set botf [frame $text.bot]
    button $botf.ok -text OK -command "action {$cmd} $canvas $type"
    button $botf.cancel -text Cancel -command "setNull $canvas"
    frame $botf.default -relief sunken -bd 1

    raise $botf.ok
    pack $botf -padx 2m
    pack $botf.default -side left -expand 1 -padx 3m -pady 2m    
    pack $botf.ok -in $botf.default -side left -padx 2m -pady 2m \
        -ipady 1m -ipadx 2m
    pack $botf.cancel -side left -pady 3m -padx 3m -ipadx 2m -ipady 1m \
        -expand 1
} 

proc delete_text_frame {text} {
    set canvas $text.c

    $canvas delete "text_item"
}

proc insert_text_frame2 {text f} {
}

proc insert_text_frame {text f} {
    set canvas $text.c
    
    delete_text_frame $text

    $canvas create window 0 0 -window $f -anchor nw \
            -tags "text_item"

    $canvas configure -scrollregion [$canvas bbox text_item]
}
    
##########################################################################
# Store the IDs of slots
##########################################################################

proc set_display_node_pos {canvas obj res} {
    global KB_Slot_ID

    set slots [lindex $res 1]
    
    set x 0
    set y 0

    foreach s $slots {
        set slot [kb name $s]
        
        if {[regexp {[ |]*x[ |]*} $slot all test]} {
            set KB_Slot_ID($canvas,$obj,x) $s
            set x [kb value $s]
        }

        if {[regexp {[ |]*y[ |]*} $slot all test]} {
            set KB_Slot_ID($canvas,$obj,y) $s
            set y [kb value $s]    
        }
    }

    return [list $x $y]
}
        
##########################################################################
# Adding Nodes From File
##########################################################################

proc display_actor_node {canvas obj {status ""} {xscale 1.0} {yscale 1.0}} {
    global Actor_No Display_Node_Pos Display_Node_ID Display_Node_Main
    
    if {$status == ""} {
    
        incr Actor_No
        set id actor$Actor_No
        set coord [set_display_node_pos $canvas $obj \
            [get_attr_val $obj coords]]
            
        create_actor_node $canvas [lindex $coord 0] [lindex $coord 1] \
            $id $obj      
            
    } else {
    
        ##### redisplay screen #####
        
        set tag $Display_Node_ID($canvas,$obj)
        set main $Display_Node_Main($canvas,$tag)
        set coord $Display_Node_Pos($canvas,$tag)
        
        $canvas scale $main [lindex $coord 0] [lindex $coord 1] $xscale $yscale
        check_adjust_canvas $canvas [lindex $coord 0] [lindex $coord 1]
    }
}

proc display_agent_node {canvas obj {status ""} {xscale 1.0} {yscale 1.0}} {
    global Agent_No Display_Node_Pos Display_Node_ID Display_Node_Main 
    global Display_Node_Line
    
    if {$status == ""} {
    
        incr Agent_No
        set id agent$Agent_No
        set coord [set_display_node_pos $canvas $obj \
            [get_attr_val $obj coords]]
            
        create_agent_node $canvas [lindex $coord 0] [lindex $coord 1] \
            $id $obj      
            
    } else {
        
        ##### redisplay screen #####
        
        set tag $Display_Node_ID($canvas,$obj)
        set main $Display_Node_Main($canvas,$tag)
        set coord $Display_Node_Pos($canvas,$tag)
        set line $Display_Node_Line($canvas,$tag)
        
        $canvas scale $main [lindex $coord 0] [lindex $coord 1] $xscale $yscale
        $canvas scale $line [lindex $coord 0] [lindex $coord 1] $xscale $yscale
        check_adjust_canvas $canvas [lindex $coord 0] [lindex $coord 1]        
    }    
}

proc display_role_node {canvas obj {status ""} {xscale 1.0} {yscale 1.0}} {
    global Role_No Display_Node_Pos Display_Node_ID Display_Node_Main
    global Display_Node_Line
        
    if {$status == ""} {
    
        incr Role_No
        set id role$Role_No
        set coord [set_display_node_pos $canvas $obj \
            [get_attr_val $obj coords]]
            
        create_role_node $canvas [lindex $coord 0] [lindex $coord 1] \
            $id $obj      

    } else {
    
        ##### redisplay screen #####
        
        set tag $Display_Node_ID($canvas,$obj)
        set main $Display_Node_Main($canvas,$tag)
        set coord $Display_Node_Pos($canvas,$tag)
        set line $Display_Node_Line($canvas,$tag)
        
        $canvas scale $main [lindex $coord 0] [lindex $coord 1] $xscale $yscale
        $canvas scale $line [lindex $coord 0] [lindex $coord 1] $xscale $yscale
        check_adjust_canvas $canvas [lindex $coord 0] [lindex $coord 1]
    }
}

proc display_position_node {canvas obj {status ""} {xscale 1.0} {yscale 1.0}} {
    global Position_No Display_Node_Pos Display_Node_ID Display_Node_Main
    global Display_Node_Line Display_Node_Line2
        
    if {$status == ""} {
    
        incr Position_No    
        set id position$Position_No
        set coord [set_display_node_pos $canvas $obj \
            [get_attr_val $obj coords]]
            
        create_position_node $canvas [lindex $coord 0] [lindex $coord 1] \
            $id $obj    
            
    } else {
    
        ##### redisplay screen #####
        ### everything with line2, Display_Node_Line2 by Mike!!!!        
        set tag $Display_Node_ID($canvas,$obj)
        set main $Display_Node_Main($canvas,$tag)
        set coord $Display_Node_Pos($canvas,$tag)
        set line $Display_Node_Line($canvas,$tag)
        set line2 $Display_Node_Line2($canvas,$tag)
        
        $canvas scale $main [lindex $coord 0] [lindex $coord 1] $xscale $yscale
        $canvas scale $line [lindex $coord 0] [lindex $coord 1] $xscale $yscale
        $canvas scale $line2 [lindex $coord 0] [lindex $coord 1] $xscale $yscale
        check_adjust_canvas $canvas [lindex $coord 0] [lindex $coord 1]
    }    
}

proc display_label_node {canvas obj {status ""} {xscale 1.0} {yscale 1.0}} {
    global Label_No Display_Node_Pos Display_Node_ID Display_Node_Main Display_Node_Line
    
    if {$status == ""} {

        incr Label_No    
        set id label$Label_No
        set contexts ""
        set coord [set_display_node_pos $canvas $obj \
            [get_attr_val $obj coords]]
        set contexts [lindex [get_attr_val $obj wrt] 0]                        
        
        if {$contexts != ""} {
    
            set cont [lindex $contexts 0]
        
            if {[info exists Display_Node_ID($canvas,$cont)]} {
        
                set con $Display_Node_ID($canvas,$cont)
                create_context_region $canvas $con
            
                create_label_node $canvas \
                    [lindex $coord 0] [lindex $coord 1] $id $obj 1 $con     
                return;
            }
        }
        create_label_node $canvas [lindex $coord 0] [lindex $coord 1] $id $obj
       
    } else {
    
        ##### redisplay screen #####
        
        set tag $Display_Node_ID($canvas,$obj)
        set main $Display_Node_Main($canvas,$tag)        
        set coord $Display_Node_Pos($canvas,$tag)
        set line $Display_Node_Line($canvas,$tag)
        
        $canvas scale $main [lindex $coord 0] [lindex $coord 1] $xscale $yscale
        $canvas scale $line [lindex $coord 0] [lindex $coord 1] $xscale $yscale
        check_adjust_canvas $canvas [lindex $coord 0] [lindex $coord 1]
    }    
}

proc display_alternative_node {canvas obj {status ""} {xscale 1.0} {yscale 1.0}} {
    global Alternative_No Display_Node_Pos Display_Node_ID Display_Node_Main
    global Display_Node_Line
    
    if {$status == ""} {

        incr Alternative_No    
        set id alternative$Alternative_No
        set contexts ""
        set coord [set_display_node_pos $canvas $obj \
            [get_attr_val $obj coords]]
        set contexts [lindex [get_attr_val $obj wrt] 0]                        
        
        if {$contexts != ""} {
    
            set cont [lindex $contexts 0]
        
            if {[info exists Display_Node_ID($canvas,$cont)]} {
        
                set con $Display_Node_ID($canvas,$cont)
                create_context_region $canvas $con
            
                create_alternative_node $canvas \
                    [lindex $coord 0] [lindex $coord 1] $id $obj 1 $con     
                return;
            }
        }
        create_alternative_node $canvas [lindex $coord 0] [lindex $coord 1] $id $obj
       
    } else {
    
        ##### redisplay screen #####
        
        set tag $Display_Node_ID($canvas,$obj)
        set main $Display_Node_Main($canvas,$tag)        
        set coord $Display_Node_Pos($canvas,$tag)
        set line $Display_Node_Line($canvas,$tag)
        
        $canvas scale $main [lindex $coord 0] [lindex $coord 1] $xscale $yscale
        $canvas scale $line [lindex $coord 0] [lindex $coord 1] $xscale $yscale
        
        check_adjust_canvas $canvas [lindex $coord 0] [lindex $coord 1]
    }    
}
 
proc display_task_node {canvas obj {status ""} {xscale 1.0} {yscale 1.0}} {
    global Task_No Display_Node_ID Display_Node_Pos Display_Node_Main Display_Node_Line

    if {$status == ""} {

        incr Task_No    
        set id task$Task_No
        set contexts ""
        set coord [set_display_node_pos $canvas $obj \
            [get_attr_val $obj coords]]
        set contexts [lindex [get_attr_val $obj wrt] 0]                        
        
        if {$contexts != ""} {
    
            set cont [lindex $contexts 0]
        
            if {[info exists Display_Node_ID($canvas,$cont)]} {
        
                set con $Display_Node_ID($canvas,$cont)
                create_context_region $canvas $con
            
                create_task_node $canvas \
                    [lindex $coord 0] [lindex $coord 1] $id $obj 1 $con     
                return;
            }
        }
        create_task_node $canvas [lindex $coord 0] [lindex $coord 1] $id $obj
       
    } else {
    
        ##### redisplay screen #####
        
        set tag $Display_Node_ID($canvas,$obj)
        set main $Display_Node_Main($canvas,$tag)        
        set coord $Display_Node_Pos($canvas,$tag)
        set line $Display_Node_Line($canvas,$tag)
        
        $canvas scale $main [lindex $coord 0] [lindex $coord 1] $xscale $yscale
        $canvas scale $line [lindex $coord 0] [lindex $coord 1] $xscale $yscale
        
        check_adjust_canvas $canvas [lindex $coord 0] [lindex $coord 1]
    }    
} 

proc display_resource_node {canvas obj {status ""} {xscale 1.0} {yscale 1.0}} {
    global Resource_No Display_Node_ID Display_Node_Pos Display_Node_Main

    if {$status == ""} {

        incr Resource_No    
        set id resource$Resource_No
        set contexts ""
        set coord [set_display_node_pos $canvas $obj \
            [get_attr_val $obj coords]]
        set contexts [lindex [get_attr_val $obj wrt] 0]                        
        
        if {$contexts != ""} {
    
            set cont [lindex $contexts 0]
        
            if {[info exists Display_Node_ID($canvas,$cont)]} {
        
                set con $Display_Node_ID($canvas,$cont)
                create_context_region $canvas $con
            
                create_resource_node $canvas \
                    [lindex $coord 0] [lindex $coord 1] $id $obj 1 $con     
                return;
            }
        }
        create_resource_node $canvas [lindex $coord 0] [lindex $coord 1] $id $obj
       
    } else {
    
        ##### redisplay screen #####
        
        set tag $Display_Node_ID($canvas,$obj)
        set main $Display_Node_Main($canvas,$tag)        
        set coord $Display_Node_Pos($canvas,$tag)
        
        $canvas scale $main [lindex $coord 0] [lindex $coord 1] $xscale $yscale
        check_adjust_canvas $canvas [lindex $coord 0] [lindex $coord 1]
    }    
}

proc display_goal_node {canvas obj {status ""} {xscale 1.0} {yscale 1.0}} {
    global Goal_No Display_Node_ID Display_Node_Pos Display_Node_Main Display_Node_Line

    if {$status == ""} {

        incr Goal_No    
        set id goal$Goal_No
        set contexts ""
        set coord [set_display_node_pos $canvas $obj \
            [get_attr_val $obj coords]]
        set contexts [lindex [get_attr_val $obj wrt] 0]                        
        
        if {$contexts != ""} {
    
            set cont [lindex $contexts 0]
        
            if {[info exists Display_Node_ID($canvas,$cont)]} {
        
                set con $Display_Node_ID($canvas,$cont)
                create_context_region $canvas $con
   
                create_goal_node $canvas \
                    [lindex $coord 0] [lindex $coord 1] $id $obj 1 $con     
                return;
            }
        }
        create_goal_node $canvas [lindex $coord 0] [lindex $coord 1] $id $obj
       
    } else {
    
        ##### redisplay screen #####
        
        set tag $Display_Node_ID($canvas,$obj)        
        set main $Display_Node_Main($canvas,$tag)        
        set coord $Display_Node_Pos($canvas,$tag)
        set line $Display_Node_Line($canvas,$tag)
        
        $canvas scale $main [lindex $coord 0] [lindex $coord 1] $xscale $yscale
        $canvas scale $line [lindex $coord 0] [lindex $coord 1] $xscale $yscale
        check_adjust_canvas $canvas [lindex $coord 0] [lindex $coord 1]
    }    
}

proc display_softgoal_node {canvas obj {status ""} {xscale 1.0} {yscale 1.0}} {
    global Softgoal_No Display_Node_ID Display_Node_Pos Display_Node_Main Display_Node_Line

    if {$status == ""} {

        incr Softgoal_No    
        set id softgoal$Softgoal_No
        set contexts ""
        set coord [set_display_node_pos $canvas $obj \
            [get_attr_val $obj coords]]
        set contexts [lindex [get_attr_val $obj wrt] 0]                        
        
        if {$contexts != ""} {
    
            set cont [lindex $contexts 0]
        
            if {[info exists Display_Node_ID($canvas,$cont)]} {
        
                set con $Display_Node_ID($canvas,$cont)
                create_context_region $canvas $con
            
                create_softgoal_node $canvas \
                    [lindex $coord 0] [lindex $coord 1] $id $obj 1 $con     
                return;
            }
        }
        create_softgoal_node $canvas [lindex $coord 0] [lindex $coord 1] $id $obj
       
    } else {
    
        ##### redisplay screen #####
        
        set tag $Display_Node_ID($canvas,$obj)
        set main $Display_Node_Main($canvas,$tag)        
        set coord $Display_Node_Pos($canvas,$tag)
        set line $Display_Node_Line($canvas,$tag)
        
        $canvas scale $main [lindex $coord 0] [lindex $coord 1] $xscale $yscale
        $canvas scale $line [lindex $coord 0] [lindex $coord 1] $xscale $yscale
        check_adjust_canvas $canvas [lindex $coord 0] [lindex $coord 1]
    }    
}

proc display_all_nodes {canvas} {
    global Exist_Object Object_List
        
    foreach type $Object_List { 
    
        ##### change the first character into small letter #####
        set tmp ""
        append tmp [string tolower [string index $type 0]] \
            [string range $type 1 end]
puts $tmp
        if {[info exists Exist_Object($canvas,$type)]} {
        
            foreach obj $Exist_Object($canvas,$type) {
                display_${tmp}_node $canvas $obj
            }
        }
    }
}    

proc display_node_link {canvas obj} {
    global Display_Node_ID

    set slots [lindex [get_attr_val $obj depend_to] 1]

    foreach s $slots {

        set slot [kb name $s]
        
        if {[regexp {^o_dependency*} $slot all test]} {
            set val [lindex [kb value $s] 0]
            
            create_depend_link $canvas \
                    $Display_Node_ID($canvas,$obj) \
                    $Display_Node_ID($canvas,$val) "o" 0           
        }
        
        if {[regexp {^x_dependency*} $slot all test]} {
            set val [lindex [kb value $s] 0]
           
            create_depend_link $canvas \
                    $Display_Node_ID($canvas,$obj) \
                    $Display_Node_ID($canvas,$val) "x" 0           
        }

        if {[regexp {^dependency*} $slot all test]} {
            set val [lindex [kb value $s] 0]

            create_depend_link $canvas \
                    $Display_Node_ID($canvas,$obj) \
                    $Display_Node_ID($canvas,$val) "" 0
        }

        if {[regexp {^task_decomposition*} $slot all test]} {
            set val [lindex [kb value $s] 0]

            create_task_decomp_link $canvas \
                    $Display_Node_ID($canvas,$obj)\
                    $Display_Node_ID($canvas,$val)
        }

        if {[regexp {^means_ends*} $slot all test]} {
            set val [lindex [kb value $s] 0]

            create_means_ends_link $canvas \
                    $Display_Node_ID($canvas,$obj) \
                    $Display_Node_ID($canvas,$val)
        }
        
        if {[regexp {^pos_softgoal*} $slot all test]} {
            set val [lindex [kb value $s] 0]

            create_softgoal_link $canvas \
                    $Display_Node_ID($canvas,$obj) \
                    $Display_Node_ID($canvas,$val) "+"
                    
        }
        
        if {[regexp {^neg_softgoal*} $slot all test]} {
            set val [lindex [kb value $s] 0]

            create_softgoal_link $canvas \
                    $Display_Node_ID($canvas,$obj) \
                    $Display_Node_ID($canvas,$val) "-"                   
        }
        
        if {[regexp {^pos_sub_softgoal*} $slot all test]} {
            set val [lindex [kb value $s] 0]

            create_softgoal_link $canvas \
                    $Display_Node_ID($canvas,$obj) \
                    $Display_Node_ID($canvas,$val) "+sub"
                    
        }
        
        if {[regexp {^neg_sub_softgoal*} $slot all test]} {
            set val [lindex [kb value $s] 0]

            create_softgoal_link $canvas \
                    $Display_Node_ID($canvas,$obj) \
                    $Display_Node_ID($canvas,$val) "-sub"   
        }

        if {[regexp {^pos_sup_softgoal*} $slot all test]} {
            set val [lindex [kb value $s] 0]

            create_softgoal_link $canvas \
                    $Display_Node_ID($canvas,$obj) \
                    $Display_Node_ID($canvas,$val) "+sup"
                    
        }
        
        if {[regexp {^neg_sup_softgoal*} $slot all test]} {
            set val [lindex [kb value $s] 0]

            create_softgoal_link $canvas \
                    $Display_Node_ID($canvas,$obj) \
                    $Display_Node_ID($canvas,$val) "-sup" 
        }

        if {[regexp {^and_softgoal*} $slot all test]} {
            set val [lindex [kb value $s] 0]

            create_softgoal_link $canvas \
                    $Display_Node_ID($canvas,$obj) \
                    $Display_Node_ID($canvas,$val) "and"
                    
        }
        
        if {[regexp {^or_softgoal*} $slot all test]} {
            set val [lindex [kb value $s] 0]

            create_softgoal_link $canvas \
                    $Display_Node_ID($canvas,$obj) \
                    $Display_Node_ID($canvas,$val) "or"   
        }

        if {[regexp {^eql_softgoal*} $slot all test]} {
            set val [lindex [kb value $s] 0]

            create_softgoal_link $canvas \
                    $Display_Node_ID($canvas,$obj) \
                    $Display_Node_ID($canvas,$val) "eql"
        }

        if {[regexp {^softgoal*} $slot all test]} {
            set val [lindex [kb value $s] 0]
            
            create_softgoal_link $canvas \
                    $Display_Node_ID($canvas,$obj) \
                    $Display_Node_ID($canvas,$val)             
        } 
        
        if {[regexp {^part*} $slot all test]} {
            set val [lindex [kb value $s] 0]

            create_part_relation $canvas \
                    $Display_Node_ID($canvas,$obj) \
                    $Display_Node_ID($canvas,$val) PART            
        } 
        
        if {[regexp {^play*} $slot all test]} {
            set val [lindex [kb value $s] 0]

            create_play_relation $canvas \
                    $Display_Node_ID($canvas,$obj) \
                    $Display_Node_ID($canvas,$val) PLAYS                  
        } 
        
        if {[regexp {^cover*} $slot all test]} {
            set val [lindex [kb value $s] 0]
      
            create_cover_relation $canvas \
                    $Display_Node_ID($canvas,$obj) \
                    $Display_Node_ID($canvas,$val) COVERS
        }
        
        if {[regexp {^isa*} $slot all test]} {
            set val [lindex [kb value $s] 0]

            create_isa_relation $canvas \
                    $Display_Node_ID($canvas,$obj) \
                    $Display_Node_ID($canvas,$val) ISA     
        }
        
        if {[regexp {^occupy*} $slot all test]} {
            set val [lindex [kb value $s] 0]

            create_occupy_relation $canvas \
                    $Display_Node_ID($canvas,$obj) \
                    $Display_Node_ID($canvas,$val) OCCUPIES
        }   
    }
}

proc display_all_links {canvas} {
    global Exist_Object Object_List
    
    foreach type $Object_List {
    
        if {[info exists Exist_Object($canvas,$type)]} {
        
            foreach obj $Exist_Object($canvas,$type) {
                display_node_link $canvas $obj
            }
        }
    }
}

proc complete_file_design {canvas file} {

    load_cfg_file $file
    display_all_nodes $canvas    
    display_all_links $canvas
}

##########################################################################
# Adding Nodes
##########################################################################

set Input_No 0

proc prompt_text_window {canvas type msg id cmd {wrt 0}} {
    global Input_No Edit_Label User_Input
    
    incr Input_No
    set User_Input(label) ""
    set User_Input(context) none

    set text [toplevel .text$Input_No]
    wm title $text $msg
    
    pack [label $text.lbl -text ""]
    
    set tmp ""
    append tmp [string tolower [string index $type 0]] \
        [string range $type 1 end]
    
    create_text_edit $canvas $type $id $text \
        "dual_action $canvas $id okay_new_$tmp {$cmd}" $wrt
    
    set oldFocus [focus]
    catch {
        grab $text
    }
    focus $text
    
    tkwait variable Edit_Label($canvas)
    destroy $text
    display_message $canvas ""
    set User_Input(label) ""
    set User_Input(context) none    
    focus $oldFocus
}

proc dual_action {canvas id {cmd0 ""} {cmd1 ""} {sign ""}} {
    if {$cmd0 != ""} {
        eval $cmd0 $canvas $id
    }

    if {$cmd1 != ""} {
    
        ####### the fifth element cmd1 is sign ######
        ####### use linsert to move the sign to the last place #####
        
        if {[llength $cmd1] == 5} {
            eval [linsert $cmd1 4 $canvas $id]
        } else {
            eval $cmd1 $canvas $id
        }    
    }
}

proc okay_new_label {canvas id} {
    global Display_Canvas_Pos Edit_Label Canvas_To_Text Display_Node_ID
    global Edit_Context Default_Pos
    
    display_message $canvas "" 
    
    if {$Edit_Label($canvas) != ""} {
    
        if {[info exists Display_Canvas_Pos($canvas)] == 0} {
            set Display_Canvas_Pos($canvas) $Default_Pos
        }
       
        set coord $Display_Canvas_Pos($canvas)
        
        add_object $canvas Label $Edit_Label($canvas)

        if {[info exists Display_Node_ID($canvas,$Edit_Context($canvas))]} {
            kb_create Label $Edit_Label($canvas) [lindex $coord 0] \
                [lindex $coord 1] $Edit_Context($canvas)

            create_label_node $canvas [lindex $coord 0] [lindex $coord 1] \
                $id $Edit_Label($canvas) 1 \
                $Display_Node_ID($canvas,$Edit_Context($canvas))
                
        } else {
            
            kb_create Label $Edit_Label($canvas) [lindex $coord 0] \
                [lindex $coord 1]
            create_label_node $canvas \
                [lindex $coord 0] [lindex $coord 1] \
                $id $Edit_Label($canvas)
        }

        unset Display_Canvas_Pos($canvas)
        set Edit_Label($canvas) ""
        set Edit_Context($canvas) ""
    }
}

proc new_label {canvas {cmd ""}} {
    global Canvas_To_Text Label_No Object_No
   
    incr Object_No

    incr Label_No
    set id "label$Label_No"

    unpost_canvas_menu $canvas
    
    prompt_text_window $canvas Label "Create new Label" $id $cmd 1     
}

proc okay_new_alternative {canvas id} {
    global Display_Canvas_Pos Edit_Label Canvas_To_Text Display_Node_ID
    global Edit_Context Default_Pos

    display_message $canvas "" 
    
    if {$Edit_Label($canvas) != ""} {
    
        if {[info exists Display_Canvas_Pos($canvas)] == 0} {
            set Display_Canvas_Pos($canvas) $Default_Pos
        }
            
        set coord $Display_Canvas_Pos($canvas)
        
        add_object $canvas Alternative $Edit_Label($canvas)

        if {[info exists Display_Node_ID($canvas,$Edit_Context($canvas))]} {
            kb_create Alternative $Edit_Label($canvas) [lindex $coord 0] \
                [lindex $coord 1] $Edit_Context($canvas)

            create_alternative_node $canvas [lindex $coord 0] [lindex $coord 1] \
                $id $Edit_Label($canvas) 1 \
                $Display_Node_ID($canvas,$Edit_Context($canvas))
                
        } else {
            
            kb_create Alternative $Edit_Label($canvas) [lindex $coord 0] \
                [lindex $coord 1]
            create_alternative_node $canvas \
                [lindex $coord 0] [lindex $coord 1] \
                $id $Edit_Label($canvas)
        }

        unset Display_Canvas_Pos($canvas)
        set Edit_Label($canvas) ""
        set Edit_Context($canvas) ""
    }
}

proc new_alternative {canvas {cmd ""}} {
    global Canvas_To_Text Alternative_No Object_No
   
    incr Object_No

    incr Alternative_No
    set id "alternative$Alternative_No"

    unpost_canvas_menu $canvas
    
    prompt_text_window $canvas Alternative "Create new Alternative" $id $cmd 1 
}

proc okay_new_actor {canvas id} {
    global Display_Canvas_Pos Edit_Label Canvas_To_Text Default_Pos

    display_message $canvas ""
    
    if {$Edit_Label($canvas) != ""} {
    
        if {[info exists Display_Canvas_Pos($canvas)] == 0} {
            set Display_Canvas_Pos($canvas) $Default_Pos
        }
        
        set coord $Display_Canvas_Pos($canvas)

        add_object $canvas Actor $Edit_Label($canvas)
    
        kb_create Actor $Edit_Label($canvas) [lindex $coord 0] \
                [lindex $coord 1]

        create_actor_node $canvas \
                [lindex $coord 0] [lindex $coord 1] \
                $id $Edit_Label($canvas)

        unset Display_Canvas_Pos($canvas)
        set Edit_Label($canvas) ""
    }
}

proc new_actor {canvas {cmd ""}} {
    global Actor_No Object_No
   
    incr Object_No
    
    incr Actor_No
    set id "actor$Actor_No"
    
    unpost_canvas_menu $canvas
    
    prompt_text_window $canvas Actor "Create new Actor" $id $cmd
}

proc okay_new_agent {canvas id} {
    global Display_Canvas_Pos Edit_Label Canvas_To_Text Default_Pos
    
    display_message $canvas ""
    
    if {$Edit_Label($canvas) != ""} {
    
        if {[info exists Display_Canvas_Pos($canvas)] == 0} {
            set Display_Canvas_Pos($canvas) $Default_Pos
        }
        
        set coord $Display_Canvas_Pos($canvas)
        
        add_object $canvas Agent $Edit_Label($canvas)
        
        kb_create Agent $Edit_Label($canvas) [lindex $coord 0] \
                [lindex $coord 1]

        create_agent_node $canvas \
                [lindex $coord 0] [lindex $coord 1] \
                $id $Edit_Label($canvas)

        unset Display_Canvas_Pos($canvas)
        set Edit_Label($canvas) ""
    }
}

proc new_agent {canvas {cmd ""}} {
    global Canvas_To_Text Agent_No Object_No
   
    incr Object_No

    incr Agent_No
    set id "agent$Agent_No"

    unpost_canvas_menu $canvas
    
    prompt_text_window $canvas Agent "Create new Agent" $id $cmd    
}

proc okay_new_role {canvas id} {
    global Display_Canvas_Pos Edit_Label Canvas_To_Text Default_Pos
    
    display_message $canvas ""
    
    if {$Edit_Label($canvas) != ""} {
    
        if {[info exists Display_Canvas_Pos($canvas)] == 0} {
            set Display_Canvas_Pos($canvas) $Default_Pos
        }
        
        set coord $Display_Canvas_Pos($canvas)
        
        add_object $canvas Role $Edit_Label($canvas)
        
        kb_create Role $Edit_Label($canvas) [lindex $coord 0] \
                [lindex $coord 1]

        create_role_node $canvas \
                [lindex $coord 0] [lindex $coord 1] \
                $id $Edit_Label($canvas)

        unset Display_Canvas_Pos($canvas)
        set Edit_Label($canvas) ""
    }
}

proc new_role {canvas {cmd ""}} {
    global Canvas_To_Text Role_No Object_No
   
    incr Object_No

    incr Role_No
    set id "role$Role_No"

    unpost_canvas_menu $canvas
    
    prompt_text_window $canvas Role "Create new Role" $id $cmd
}

proc okay_new_position {canvas id} {
    global Display_Canvas_Pos Edit_Label Canvas_To_Text Default_Pos

    display_message $canvas ""
    
    if {$Edit_Label($canvas) != ""} {
    
        if {[info exists Display_Canvas_Pos($canvas)] == 0} {
            set Display_Canvas_Pos($canvas) $Default_Pos
        }
        
        set coord $Display_Canvas_Pos($canvas)
        
        add_object $canvas Position $Edit_Label($canvas)

        kb_create Position $Edit_Label($canvas) [lindex $coord 0] \
                [lindex $coord 1]

        create_position_node $canvas \
                [lindex $coord 0] [lindex $coord 1] \
                $id $Edit_Label($canvas)

        unset Display_Canvas_Pos($canvas)
        set Edit_Label($canvas) ""
    }
}

proc new_position {canvas {cmd ""}} {
    global Canvas_To_Text Position_No Object_No
   
    incr Object_No

    incr Position_No
    set id "position$Position_No"

    unpost_canvas_menu $canvas
    
    prompt_text_window $canvas Position "Create new Position" $id $cmd     
}

proc okay_new_task {canvas id} {
    global Display_Canvas_Pos Edit_Label Canvas_To_Text Edit_Context
    global Display_Node_ID Default_Pos

    display_message $canvas ""
    
    if {$Edit_Label($canvas) != ""} {
    
        if {[info exists Display_Canvas_Pos($canvas)] == 0} {
            set Display_Canvas_Pos($canvas) $Default_Pos
        }
        
        set coord $Display_Canvas_Pos($canvas)
        
        add_object $canvas Task $Edit_Label($canvas)

        if {[info exists Display_Node_ID($canvas,$Edit_Context($canvas))]} {
            kb_create Task $Edit_Label($canvas) [lindex $coord 0] \
                [lindex $coord 1] $Edit_Context($canvas)

            create_task_node $canvas \
                [lindex $coord 0] [lindex $coord 1] \
                $id $Edit_Label($canvas) 1 \
                $Display_Node_ID($canvas,$Edit_Context($canvas))
                
        } else {
         
            kb_create Task $Edit_Label($canvas) [lindex $coord 0] \
                [lindex $coord 1]
            create_task_node $canvas \
                [lindex $coord 0] [lindex $coord 1] \
                $id $Edit_Label($canvas)
        }
        
        unset Display_Canvas_Pos($canvas)
        set Edit_Label($canvas) ""
        set Edit_Context($canvas) ""
    }
}

proc new_task {canvas {cmd ""}} {
    global Canvas_To_Text Task_No Object_No
   
    incr Object_No

    incr Task_No
    set id "task$Task_No"

    unpost_canvas_menu $canvas
    
    prompt_text_window $canvas Task "Create new Task" $id $cmd 1 
}

proc okay_new_resource {canvas id} {
    global Display_Canvas_Pos Edit_Label Canvas_To_Text Edit_Context
    global Display_Node_ID Default_Pos

    display_message $canvas ""
    
    if {$Edit_Label($canvas) != ""} {
    
        if {[info exists Display_Canvas_Pos($canvas)] == 0} {
            set Display_Canvas_Pos($canvas) $Default_Pos
        }
       
        set coord $Display_Canvas_Pos($canvas)
        
        add_object $canvas Resource $Edit_Label($canvas)
        
        if {[info exists Display_Node_ID($canvas,$Edit_Context($canvas))]} {
            kb_create Resource $Edit_Label($canvas) [lindex $coord 0] \
                [lindex $coord 1] $Edit_Context($canvas)
                
            create_resource_node $canvas \
                [lindex $coord 0] [lindex $coord 1] \
                $id $Edit_Label($canvas) 1 \
                $Display_Node_ID($canvas,$Edit_Context($canvas))
                
        } else {
        
            kb_create Resource $Edit_Label($canvas) [lindex $coord 0] \
                [lindex $coord 1]
            create_resource_node $canvas \
                [lindex $coord 0] [lindex $coord 1] \
                $id $Edit_Label($canvas)
        }
        
        unset Display_Canvas_Pos($canvas)
        set Edit_Label($canvas) ""
        set Edit_Context($canvas) ""
    }
}

proc new_resource {canvas {cmd ""}} {
    global Canvas_To_Text Resource_No Object_No

    incr Object_No
    incr Resource_No
    set id "res$Resource_No"

    unpost_canvas_menu $canvas
    
    prompt_text_window $canvas Resource "Create new Resource" $id $cmd 1
}

proc okay_new_goal {canvas id} {
    global Display_Canvas_Pos Edit_Label Canvas_To_Text Edit_Context
    global Display_Node_ID Default_Pos

    display_message $canvas ""
    
    if {$Edit_Label($canvas) != ""} {
    
        if {[info exists Display_Canvas_Pos($canvas)] == 0} {
            set Display_Canvas_Pos($canvas) $Default_Pos
        }
         
        set coord $Display_Canvas_Pos($canvas)
        
        add_object $canvas Goal $Edit_Label($canvas)
        
        if {[info exists Display_Node_ID($canvas,$Edit_Context($canvas))]} {
            kb_create Goal $Edit_Label($canvas) [lindex $coord 0] \
                [lindex $coord 1] $Edit_Context($canvas)
            create_goal_node $canvas \
                [lindex $coord 0] [lindex $coord 1] \
                $id $Edit_Label($canvas) 1 \
                $Display_Node_ID($canvas,$Edit_Context($canvas))
                
        } else {

            kb_create Goal $Edit_Label($canvas) [lindex $coord 0] \
                [lindex $coord 1] 
            create_goal_node $canvas \
                [lindex $coord 0] [lindex $coord 1] \
                $id $Edit_Label($canvas)
        }
        
        unset Display_Canvas_Pos($canvas)
        set Edit_Label($canvas) ""
        set Edit_Context($canvas) ""
    }
}

proc new_goal {canvas {cmd ""}} {
    global Canvas_To_Text Goal_No Object_No

    incr Object_No
    incr Goal_No
    set id "goal$Goal_No"

    unpost_canvas_menu $canvas
    
    prompt_text_window $canvas Goal "Create new Goal" $id $cmd 1
}

proc okay_new_softgoal {canvas id} {
    global Display_Canvas_Pos Edit_Label Canvas_To_Text Edit_Context
    global Display_Node_ID Default_Pos

    display_message $canvas ""
    
    if {$Edit_Label($canvas) != ""} {
    
        if {[info exists Display_Canvas_Pos($canvas)] == 0} {
            set Display_Canvas_Pos($canvas) $Default_Pos
        }
        
        set coord $Display_Canvas_Pos($canvas)
        
        add_object $canvas Softgoal $Edit_Label($canvas)

        if {[info exists Display_Node_ID($canvas,$Edit_Context($canvas))]} {
            kb_create Softgoal $Edit_Label($canvas) [lindex $coord 0] \
                [lindex $coord 1] $Edit_Context($canvas)

            create_softgoal_node $canvas \
                [lindex $coord 0] [lindex $coord 1] \
                $id $Edit_Label($canvas) 1 \
                $Display_Node_ID($canvas,$Edit_Context($canvas))
                
        } else {
        
            kb_create Softgoal $Edit_Label($canvas) [lindex $coord 0] \
                [lindex $coord 1]

            create_softgoal_node $canvas \
                [lindex $coord 0] [lindex $coord 1] \
                $id $Edit_Label($canvas)
        }
        
        unset Display_Canvas_Pos($canvas)
        set Edit_Label($canvas) ""
        set Edit_Context($canvas) ""
    }
}

proc new_softgoal {canvas {cmd ""}} {
    global Canvas_To_Text Softgoal_No Object_No
   
    incr Object_No

    incr Softgoal_No
    set id "sgoal$Softgoal_No"

    unpost_canvas_menu $canvas
    
    prompt_text_window $canvas Softgoal "Create new Softgoal" $id $cmd 1
}

##########################################################################
# Creating Relationship
##########################################################################

proc relation_first_message {canvas} {
    display_message $canvas "From Object"
}

proc relation_second_message {canvas} {
    display_message $canvas "To Object"
}

proc okay_new_relation {canvas tag1 tag2 {sign ""}} {
    global Display_Node_Text

    display_message $canvas ""
         
    if {$sign == "PART"} {        
    
         kb_add part [lindex $Display_Node_Text($canvas,$tag1) 1]\
             [lindex $Display_Node_Text($canvas,$tag2) 1]
         create_part_relation $canvas $tag1 $tag2 "PART"
         
    } elseif {$sign == "PLAYS"} {
    
        kb_add play [lindex $Display_Node_Text($canvas,$tag1) 1]\
            [lindex $Display_Node_Text($canvas,$tag2) 1]
        create_play_relation $canvas $tag1 $tag2 "PLAYS"
        
    } elseif {$sign == "COVERS"} {
    
        kb_add cover [lindex $Display_Node_Text($canvas,$tag1) 1]\
            [lindex $Display_Node_Text($canvas,$tag2) 1]
        create_cover_relation $canvas $tag1 $tag2 "COVERS"
        
    } elseif {$sign == "ISA"} {
    
        kb_add isa [lindex $Display_Node_Text($canvas,$tag1) 1]\
            [lindex $Display_Node_Text($canvas,$tag2) 1]
        create_isa_relation $canvas $tag1 $tag2 "ISA"
        
    } elseif {$sign == "OCCUPIES"} {
    
        kb_add occupy [lindex $Display_Node_Text($canvas,$tag1) 1]\
            [lindex $Display_Node_Text($canvas,$tag2) 1]
        create_occupy_relation $canvas $tag1 $tag2 "OCCUPIES"
    }
}

proc new_relation {canvas {sign ""}} {
    unpost_canvas_menu $canvas
    
    additional_link $canvas "okay_new_relation" "relation_first_message" \
        "relation_second_message" $sign
}

##########################################################################
# Creating Links
##########################################################################

proc display_message {canvas t} {
    global Canvas_To_Text 

    if {[info exists Canvas_To_Text($canvas)]} {
        set text $Canvas_To_Text($canvas)

        if {[winfo exists $text.label]} {
            if {$t != ""} {
                $text.label configure -text "Please select the $t"
            } else {
                $text.label configure -text ""
            }
        }
    }
}

proc task_first_message {canvas} {
    display_message $canvas "Main Task"
}

proc task_second_message {canvas} {
    display_message $canvas "Decomposed Task"
}

proc okay_new_task_link {canvas tag1 tag2 {sign ""}} {
    global Display_Node_Text

    display_message $canvas ""

    kb_add task_decomposition [lindex $Display_Node_Text($canvas,$tag1) 1] \
            [lindex $Display_Node_Text($canvas,$tag2) 1]

    create_task_decomp_link $canvas $tag1 $tag2
}

proc new_task_decomp_link {canvas} {
    unpost_canvas_menu $canvas

    additional_link $canvas "okay_new_task_link" "task_first_message" \
        "task_second_message"
}

proc means_first_message {canvas} {
    display_message $canvas "End"
}

proc means_second_message {canvas} {
    display_message $canvas "Means"
}

proc okay_new_means_link {canvas tag1 tag2 {sign ""}} {
    global Display_Node_Text

    display_message $canvas ""

    kb_add means_ends [lindex $Display_Node_Text($canvas,$tag1) 1]\
            [lindex $Display_Node_Text($canvas,$tag2) 1]

    create_means_ends_link $canvas $tag1 $tag2
}

proc new_means_ends_link {canvas} {
    unpost_canvas_menu $canvas

    additional_link $canvas "okay_new_means_link" "means_first_message" \
        "means_second_message"
}

proc softgoal_first_message {canvas} {
    display_message $canvas "Achieved SoftGoal"
}

proc softgoal_second_message {canvas} {
    display_message $canvas "Contribution Element"
}

proc okay_new_softgoal_link {canvas tag1 tag2 {sign ""}} {
    global Display_Node_Text

    display_message $canvas ""
         
    if {$sign == ""} {        
    
         kb_add softgoal [lindex $Display_Node_Text($canvas,$tag1) 1]\
             [lindex $Display_Node_Text($canvas,$tag2) 1]
         create_softgoal_link $canvas $tag1 $tag2
         
    } elseif {$sign == "+"} {
    
        kb_add pos_softgoal [lindex $Display_Node_Text($canvas,$tag1) 1]\
            [lindex $Display_Node_Text($canvas,$tag2) 1]
        create_softgoal_link $canvas $tag1 $tag2 "+"
        
    } elseif {$sign == "-"} {
    
        kb_add neg_softgoal [lindex $Display_Node_Text($canvas,$tag1) 1]\
            [lindex $Display_Node_Text($canvas,$tag2) 1]
        create_softgoal_link $canvas $tag1 $tag2 "-"
        
    } elseif {$sign == "+sub"} {
    
        kb_add pos_sub_softgoal [lindex $Display_Node_Text($canvas,$tag1) 1]\
            [lindex $Display_Node_Text($canvas,$tag2) 1]
        create_softgoal_link $canvas $tag1 $tag2 "+sub"
        
    } elseif {$sign == "-sub"} {
    
        kb_add neg_sub_softgoal [lindex $Display_Node_Text($canvas,$tag1) 1]\
            [lindex $Display_Node_Text($canvas,$tag2) 1]
        create_softgoal_link $canvas $tag1 $tag2 "-sub"
        
    } elseif {$sign == "+sup"} {
    
        kb_add pos_sup_softgoal [lindex $Display_Node_Text($canvas,$tag1) 1]\
            [lindex $Display_Node_Text($canvas,$tag2) 1]
        create_softgoal_link $canvas $tag1 $tag2 "+sup"
        
    } elseif {$sign == "-sup"} {
    
        kb_add neg_sup_softgoal [lindex $Display_Node_Text($canvas,$tag1) 1]\
            [lindex $Display_Node_Text($canvas,$tag2) 1]
        create_softgoal_link $canvas $tag1 $tag2 "-sup"
        
    } elseif {$sign == "and"} {
    
        kb_add and_softgoal [lindex $Display_Node_Text($canvas,$tag1) 1]\
            [lindex $Display_Node_Text($canvas,$tag2) 1]
        create_softgoal_link $canvas $tag1 $tag2 "and"
        
    } elseif {$sign == "or"} {
    
        kb_add or_softgoal [lindex $Display_Node_Text($canvas,$tag1) 1]\
            [lindex $Display_Node_Text($canvas,$tag2) 1]
        create_softgoal_link $canvas $tag1 $tag2 "or"
        
    } elseif {$sign == "eql"} {
    
        kb_add eql_softgoal [lindex $Display_Node_Text($canvas,$tag1) 1]\
            [lindex $Display_Node_Text($canvas,$tag2) 1]
        create_softgoal_link $canvas $tag1 $tag2 "eql"
    }
}

proc new_softgoal_link {canvas {sign ""}} {
    unpost_canvas_menu $canvas

    additional_link $canvas "okay_new_softgoal_link" "softgoal_first_message" \
        "softgoal_second_message" $sign
}

##########################################################################
# Dependency Links
##########################################################################

proc okay_new_dep_link {canvas tag1 tag2 canvas2 tag3 {sign ""}} {
    global Display_Node_Text

    if {$canvas == $canvas2} {
    
        if {$sign != ""} {
            kb_add ${sign}_dependency [lindex $Display_Node_Text($canvas,$tag1) 1] \
            [lindex $Display_Node_Text($canvas,$tag3) 1]
        } else {
            kb_add dependency [lindex $Display_Node_Text($canvas,$tag1) 1] \
            [lindex $Display_Node_Text($canvas,$tag3) 1]
        }    
        
        kb_add dependency [lindex $Display_Node_Text($canvas,$tag3) 1] \
            [lindex $Display_Node_Text($canvas,$tag2) 1]

        create_depend_link $canvas $tag1 $tag3 $sign 0
        create_depend_link $canvas $tag3 $tag2 "" 0
    } else {
        puts "Fatal Error!"
        destroy .
    }
}

proc okay_new_one_sided_dep_link {canvas tag1 tag2 {sign ""}} {
    global Display_Node_Text

        if {$sign != ""} {
            kb_add ${sign}_dependency [lindex $Display_Node_Text($canvas,$tag1) 1] \
            [lindex $Display_Node_Text($canvas,$tag2) 1]
        } else {
            kb_add dependency [lindex $Display_Node_Text($canvas,$tag1) 1] \
            [lindex $Display_Node_Text($canvas,$tag2) 1]
        }    
        
        create_depend_link $canvas $tag1 $tag2 $sign 0
}

proc get_dependency_coord {canvas tag1 tag2} {
    global Display_Node_Pos

    set one $Display_Node_Pos($canvas,$tag1)
    set two $Display_Node_Pos($canvas,$tag2)

    return [list [expr ([lindex $one 0]+[lindex $two 0])/2] \
             [expr ([lindex $one 1]+[lindex $two 1])/2]]
}

proc taskdep_first_message {canvas} {
    display_message $canvas "Task Depender"
}

proc taskdep_second_message {canvas} {
    display_message $canvas "Task Dependendee"
}

proc okay_new_task_dependency {canvas tag1 tag2 {sign ""}} {
    global Display_Canvas_Pos

    set Display_Canvas_Pos($canvas) [get_dependency_coord $canvas $tag1 $tag2]

    new_task $canvas "okay_new_dep_link $canvas $tag1 $tag2 $sign"
}

proc new_task_dependency {canvas {sign ""}} {
    unpost_canvas_menu $canvas

    additional_link $canvas "okay_new_task_dependency" \
        "taskdep_first_message" \
        "taskdep_second_message" $sign
}

proc goaldep_first_message {canvas} {
    display_message $canvas "Goal Depender"
}

proc goaldep_second_message {canvas} {
    display_message $canvas "Goal Dependee"
}

proc okay_new_goal_dependency {canvas tag1 tag2 {sign ""}} {
    global Display_Canvas_Pos

    set Display_Canvas_Pos($canvas) [get_dependency_coord $canvas $tag1 $tag2]

    new_goal $canvas "okay_new_dep_link $canvas $tag1 $tag2 $sign"
}

proc new_goal_dependency {canvas {sign ""}} {
    unpost_canvas_menu $canvas
    additional_link $canvas "okay_new_goal_dependency" \
        "goaldep_first_message" \
        "goaldep_second_message" $sign
}

proc resdep_first_message {canvas} {
    display_message $canvas "Resource Depender"
}

proc resdep_second_message {canvas} {
    display_message $canvas "Resource Dependee"
}


proc okay_new_res_dependency {canvas tag1 tag2 {sign ""}} {
    global Display_Canvas_Pos

    set Display_Canvas_Pos($canvas) [get_dependency_coord $canvas $tag1 $tag2]

    new_resource $canvas "okay_new_dep_link $canvas $tag1 $tag2 $sign"
}

proc new_resource_dependency {canvas {sign ""}} {
    unpost_canvas_menu $canvas

    additional_link $canvas "okay_new_res_dependency" \
        "resdep_first_message" \
        "resdep_second_message" $sign
}

proc sgoaldep_first_message {canvas} {
    display_message $canvas "SoftGoal Depender"
}

proc sgoaldep_second_message {canvas} {
    display_message $canvas "SoftGoal Dependee"
}


proc okay_new_sgoal_dependency {canvas tag1 tag2 {sign ""}} {
    global Display_Canvas_Pos
    
    set Display_Canvas_Pos($canvas) [get_dependency_coord $canvas $tag1 $tag2]
    new_softgoal $canvas "okay_new_dep_link $canvas $tag1 $tag2 $sign"
}

proc new_softgoal_dependency {canvas {sign ""}} {
    unpost_canvas_menu $canvas

    additional_link $canvas "okay_new_sgoal_dependency" \
        "sgoaldep_first_message" \
        "sgoaldep_second_message" $sign
}

proc one_sided_first_message {canvas} {
    display_message $canvas "begin object"
}

proc one_sided_second_message {canvas} {
    display_message $canvas "end object"
}

proc okay_new_one_sided_dependency {canvas tag1 tag2 {sign ""}} {
    global Display_Canvas_Pos
    
    set Display_Canvas_Pos($canvas) [get_dependency_coord $canvas $tag1 $tag2]

    display_message $canvas ""
    okay_new_one_sided_dep_link $canvas $tag1 $tag2 $sign
}

proc one_sided_dependency {canvas {sign ""}} {
    unpost_canvas_menu $canvas
    
    additional_link $canvas "okay_new_one_sided_dependency" \
        "one_sided_first_message" \
        "one_sided_second_message" $sign
}

##########################################################################
# Setup the background menu
##########################################################################

proc post_canvas_menu {canvas x y} {
    global Display_Canvas_Menu Display_Canvas_Pos

    set Display_Canvas_Pos($canvas) "$x $y"

    catch {
        set menu $Display_Canvas_Menu($canvas)
        focus $menu
        $menu post \
            [expr $x + [winfo rootx $canvas]] \
            [expr $y + [winfo rooty $canvas]]
        grab $canvas
    } msg
}

proc unpost_canvas_menu {canvas} {
    global Display_Canvas_Menu
    catch {
        set menu $Display_Canvas_Menu($canvas)
        $menu unpost
        grab release $canvas
        focus $canvas
    }
}

proc bind_background {canvas bg} {
    $canvas bind $bg <ButtonPress-3> "
        post_canvas_menu $canvas %x %y
    "

    $canvas bind $bg <ButtonRelease-3> "
        unpost_canvas_menu $canvas
    "
}

proc setup_canvas_menu {canvas} {
    global Display_Canvas_Menu

    set menu [menu $canvas.b1_menu -tearoff n]

    $menu add command -label "Dismiss" -command "unpost_canvas_menu $canvas"
    $menu add separator

    edit_menu $menu $canvas
    
    set Display_Canvas_Menu($canvas) $menu
}

proc get_canvas_background_size {canvas coord} {
    set reqx [winfo reqwidth $canvas]
    set reqy [winfo reqheight $canvas]

    return [list \
        [expr [lindex $coord 0] < 0 ? \
          [expr [lindex $coord 0]-10] : -10] \
        [expr [lindex $coord 1] < 0 ? \
          [expr [lindex $coord 1]-10] : -10] \
        [expr [lindex $coord 2] > $reqx ? \
          [expr [lindex $coord 2]+10] : [expr $reqx+10]] \
        [expr [lindex $coord 3] > $reqy ? \
          [expr [lindex $coord 3]+10] : [expr $reqy+10]]]
}

proc lower_canvas_background {canvas} {
    global Display_Background

    set bg $Display_Background($canvas)

    $canvas lower $bg
}

proc setup_canvas_background {canvas} {
    global Display_Background Display_Generic

    set background $Display_Generic(background)

    set coord [$canvas bbox item]

    if {$coord != ""} {
        set cod [get_canvas_background_size $canvas $coord]
    } else {
        set cod [get_canvas_background_size $canvas "0 0 0 0"]
    }

    set bg [$canvas create rectangle \
        [lindex $cod 0] [lindex $cod 1] \
        [lindex $cod 2] [lindex $cod 3] \
        -outline $background -width 0 -fill $background -tag "bg$canvas"]

    $canvas lower $bg
    set Display_Background($canvas) $bg

    bind_background $canvas $bg

    setup_canvas_menu $canvas
}

proc check_adjust_canvas {canvas x y} {
    global Display_Canvas_Size

    if {[info exists Display_Canvas_Size($canvas)]} {
        set size $Display_Canvas_Size($canvas)

        if {[expr $x < [lindex $size 0]] || [expr $x > [lindex $size 2]]} {
            adjust_canvas $canvas
        } else {
            if {[expr $y < [lindex $size 1]] \
                    || [expr $y > [lindex $size 3]]} {
                adjust_canvas $canvas
            }
        }
    } else {
        adjust_canvas $canvas
    }
}
proc adjust_canvas {canvas} {
    global Display_Background Display_Canvas_Size

    set bbox [$canvas bbox all]

    if {$bbox != ""} {
        set Display_Canvas_Size($canvas) $bbox

        $canvas configure -scrollregion $bbox

        if {[info exists Display_Background($canvas)]} {
            set bg $Display_Background($canvas)

            set cod [get_canvas_background_size $canvas $bbox]

            $canvas coord $bg \
                [lindex $cod 0] [lindex $cod 1] \
                [lindex $cod 2] [lindex $cod 3]

            $canvas lower $bg
        }
    } else {
        set Display_Canvas_Size($canvas) "0 0 0 0"
    }
}

##########################################################################
# Print Utility
##########################################################################

proc print_file {canvas file orient} {
    global Display_Background Display_Generic 
    
    set bg $Display_Background($canvas)
    set background $Display_Generic(background)
    set bbox [$canvas bbox all]

    $canvas itemconfigure $bg -fill white 
    
    if {$orient == "portrait"} {
        $canvas postscript -colormode gray -file $file \
            -pagewidth 8i -pageheight 11i \
            -height [expr [lindex $bbox 3]-[lindex $bbox 1]] \
            -width [expr [lindex $bbox 2]-[lindex $bbox 0]] \
            -x [lindex $bbox 0] -y [lindex $bbox 1]
    } else {
        $canvas postscript -colormode gray -file $file \
        -pagewidth 8i -pageheight 11i -rotate y \
        -height [expr [lindex $bbox 3]-[lindex $bbox 1]] \
        -width [expr [lindex $bbox 2]-[lindex $bbox 0]] \
        -x [lindex $bbox 0] -y [lindex $bbox 1]
    }    
    
    $canvas itemconfigure $bg -fill $background
}

proc print_box {canvas orient} {
        global fsBox
        
        set fsBox(pattern) *.ps
        FSBox "Printing File"
        print_file $canvas $fsBox(path)/$fsBox(name) $orient
}
    
##########################################################################
# Help Routines
##########################################################################

proc create_help_box {win title file} {

    toplevel $win
    wm title $win $title
    wm geometry $win 50x30 
    frame $win.top -relief raised -bd 1
    pack $win.top -side top -fill both -expand 1
    frame $win.bot -relief raised -bd 1
    pack $win.bot -side bottom -fill both
    
    listbox $win.top.listbox -relief raised -borderwidth 2 \
        -yscrollcommand "$win.top.yscroll set" -setgrid 1
    pack $win.top.listbox -side left -fill both -expand 1
    
    scrollbar $win.top.yscroll -command "$win.top.listbox yview"
    pack $win.top.yscroll -side right -fill y
    
    set f [open $file r]
    while {[gets $f line] >= 0} {
        $win.top.listbox insert end $line
    }
    close $f
    
    button $win.bot.done -text Done -command "destroy $win"
    pack $win.bot.done
}

proc ome_overview {} {    
    create_help_box .overview "OME Overview" "./OME_ome.txt"    
}

proc ome_readme {} {
    create_help_box .readme "OME Readme" "./OME_readme.txt" 
}

proc ome_bugs {} {
    create_help_box .bugs "OME Bugs" "./OME_bugs.txt" 
}

proc ome_unimplemented {} {
    create_help_box .unimplem "OME Unimplemented" "./OME_unimplem.txt" 
}

##########################################################################
# Submenu Routines
##########################################################################

proc edit_menu {menu canvas} {

    $menu add command -label "Context Label" -command \
        "new_label $canvas"
    $menu add command -label "Alternative" -command \
        "new_alternative $canvas"
    $menu add separator

    $menu add command -label "Goal" -command "new_goal $canvas"
    $menu add command -label "Task" -command "new_task $canvas"
    $menu add command -label "Resource" -command "new_resource $canvas"
    $menu add command -label "SoftGoal" -command "new_softgoal $canvas"
    $menu add separator

    $menu add command -label "Actor" -command "new_actor $canvas"
    $menu add command -label "Role" -command "new_role $canvas"
    $menu add command -label "Position" -command "new_position $canvas"    
    $menu add command -label "Agent" -command "new_agent $canvas"
    $menu add separator

    $menu add cascade -label "Goal Dependency" -menu $menu.goal_dep
    $menu add cascade -label "Task Dependency" -menu $menu.task_dep         
    $menu add cascade -label "Resource Dependency" -menu $menu.res_dep
    $menu add cascade -label "Softgoal Dependency" -menu $menu.soft_dep
    $menu add cascade -label "One-sided Dependency" -menu $menu.one_sided_dep
    $menu add separator

    $menu add command -label "Task Decomposition Link" -command \
            "new_task_decomp_link $canvas"
    $menu add command -label "Means Ends Link" -command \
            "new_means_ends_link $canvas"
    $menu add cascade -label "SoftGoal Contribution Link" \
            -menu $menu.soft_link
    $menu add separator
    
    $menu add command -label "PART Relationship" -command \
            "new_relation $canvas PART"
    $menu add command -label "PLAYS Relationship" -command \
            "new_relation $canvas PLAYS"
    $menu add command -label "COVERS Relationship" -command \
            "new_relation $canvas COVERS"
    $menu add command -label "ISA Relationship" -command \
            "new_relation $canvas ISA"
    $menu add command -label "OCCUPIES Relationship" -command \
            "new_relation $canvas OCCUPIES"        
                        
    ################ New Task Dependency Submenu ################
    
    menu $menu.task_dep -tearoff n
    
    $menu.task_dep add command -label "Committed" -command \
            "new_task_dependency $canvas"
            
    $menu.task_dep add command -label "Open" -command \
            "new_task_dependency $canvas o"
            
    $menu.task_dep add command -label "Critical" -command \
            "new_task_dependency $canvas x"           
          
    ################ New Goal Dependency Submenu ################
    
    menu $menu.goal_dep -tearoff n    
    
    $menu.goal_dep add command -label "Committed" -command \
            "new_goal_dependency $canvas"
            
    $menu.goal_dep add command -label "Open" -command \
            "new_goal_dependency $canvas o"
            
    $menu.goal_dep add command -label "Critical" -command \
            "new_goal_dependency $canvas x"           
            
    ################ New Resource Dependency Submenu ################
    
    menu $menu.res_dep -tearoff n
    
    $menu.res_dep add command -label "Committed" -command \
            "new_resource_dependency $canvas"
            
    $menu.res_dep add command -label "Open" -command \
            "new_resource_dependency $canvas o"
            
    $menu.res_dep add command -label "Critical" -command \
            "new_resource_dependency $canvas x"   
            
    ################ New Softgoal Dependency Submenu ################
    
    menu $menu.soft_dep -tearoff n
    
    $menu.soft_dep add command -label "Committed" -command \
            "new_softgoal_dependency $canvas"
            
    $menu.soft_dep add command -label "Open" -command \
            "new_softgoal_dependency $canvas o"
            
    $menu.soft_dep add command -label "Critical" -command \
            "new_softgoal_dependency $canvas x"           
         
    ################ New One-sided Dependency Submenu ################
    
    menu $menu.one_sided_dep -tearoff n
    
    $menu.one_sided_dep add command -label "Committed" -command \
            "one_sided_dependency $canvas"
            
    $menu.one_sided_dep add command -label "Open" -command \
            "one_sided_dependency $canvas o"
            
    $menu.one_sided_dep add command -label "Critical" -command \
            "one_sided_dependency $canvas x"           
         
    ################ Softgoal Contribution Link Submenu ################
    
    menu $menu.soft_link -tearoff n
    
    $menu.soft_link add command -label "und" -command \
            "new_softgoal_link $canvas"
            
    $menu.soft_link add command -label "pos" -command \
            "new_softgoal_link $canvas +"
            
    $menu.soft_link add command -label "neg" -command \
            "new_softgoal_link $canvas -"           
          
    $menu.soft_link add command -label "+sub" -command \
            "new_softgoal_link $canvas +sub"
            
    $menu.soft_link add command -label "-sub" -command \
            "new_softgoal_link $canvas -sub"
            
    $menu.soft_link add command -label "+sup" -command \
            "new_softgoal_link $canvas +sup"           
            
    $menu.soft_link add command -label "-sup" -command \
            "new_softgoal_link $canvas -sup"
            
    $menu.soft_link add command -label "and" -command \
            "new_softgoal_link $canvas and"
            
    $menu.soft_link add command -label "or" -command \
            "new_softgoal_link $canvas or"           
            
    $menu.soft_link add command -label "eql" -command \
            "new_softgoal_link $canvas eql"
}

proc quit {} {
    global dlgButton

    dialogBox .dlg "Exit OME" "Are you sure to exit OME?\n" 0 Yes No
    if {$dlgButton == 0} {
        destroy .
    }
}

proc new_window {{name Guest}} {
    set res [setup_main_window $name]
    setup_draw_area [lindex $res 1]
}

proc clearGraph {canvas {status 0}} {
    global dlgButton 
   global Display_Symbology Display_Size Display_Node_Fill 
   global Display_Node_Pos Display_Nodes Display_Node_Tag
   global Display_Node_Text Display_Node_ID Display_Node_Context
   global Display_Font Font_Type Display_Node_Line
   global Display_Node_Main Display_Generic Object_No
    global Display_Font_Tag

    dialogBox .dlg "Clear Design" \
        "Are you sure to clear entire design?" 1 Yes No
        
    if {$dlgButton == 0} {
	#unset Display_Symbology 
	#unset Display_Size 
	#set Display_Node_Fill 0 
	set Display_Node_Pos(1,1) 0
	unset Display_Node_Pos
	set Display_Nodes(1) 0
	unset Display_Nodes 
	set Display_Node_Tag(1) 0
	unset Display_Node_Tag 
	set Display_Node_Text(1) 0
	unset Display_Node_Text
	set Display_Node_ID(1) 0
	unset Display_Node_ID  
	set Display_Node_Context(1) 0
	unset Display_Node_Context 
	#unset Display_Font 
	#unset Font_Type 
	set Display_Node_Line(1) 0
	unset Display_Node_Line 
	set Display_Node_Main(1) 0
	unset Display_Node_Main 
	#unset Display_Generic 
	#unset Object_No
	set Display_Font_Tag(1,1) 0
	unset Display_Font_Tag
	source OME_global.tcl
	$canvas delete all
puts $canvas
    .win1.f.frame0.c configure \
            -relief raised -width 900 -height 600 \
            -scrollregion {0c 0c 60c 48c}


	#setup_draw_area [lindex $res 1]
        #destroy $parent
        #new_window
    }
}

proc redraw_node {obj canvas xscale yscale} {
    global Exist_Object Object_List Display_Size Display_Node_ID Display_Node_Text
        
    foreach i $Object_List {
    
        set tmp ""
        append tmp [string tolower [string index $i 0]] \
            [string range $i 1 end]
            
        if {[info exists Exist_Object($canvas,$i)] && $obj == $i} {
        
            foreach j $Exist_Object($canvas,$i) {
                display_${tmp}_node $canvas $j redisplay $xscale $yscale
                
                set tag $Display_Node_ID($canvas,$j)
                set txt [lindex $Display_Node_Text($canvas,$tag) 0]
                
                $canvas itemconfigure $txt -width [expr $Display_Size($i)*1.75]
            }           
        }
    }
}    

proc redraw_font {obj canvas font} {
    global Exist_Object Font_Type Display_Node_ID Display_Node_Text Object_List
    global Display_Size Display_Font_Tag
    
    foreach i $Object_List {
            
        if {[info exists Exist_Object($canvas,$i)] && $obj == $i} {
        
            foreach j $Exist_Object($canvas,$i) {
                set tag $Display_Node_ID($canvas,$j)
#		puts $i
#		puts $tag
		if {[info exists Display_Font_Tag($tag)]} {
		unset Display_Font_Tag($tag)
		}
                set txt [lindex $Display_Node_Text($canvas,$tag) 0]
                
                $canvas itemconfigure $txt -font $Font_Type(${font}_font) \
                    -width [expr $Display_Size($i)*1.75]
            }
            
        } 
    }    
}

proc changeFont {obj font canvas} {
   global Display_Font Font_Type Object_List Display_Font_Tag
   
    if {$obj == "AllNodes"} {
	foreach obj $Object_List {
	    set Display_Font(Node,$obj) $Font_Type(${font}_font)
	    set Display_Font($obj) $Font_Type(${font}_font)
	    redraw_font $obj $canvas $font    
	}
    } else {
   set Display_Font(Node,$obj) $Font_Type(${font}_font)
   
   redraw_font $obj $canvas $font
    }
}

proc changeSize {obj size canvas} {
    global Display_Size Size_Type Object_List

    if {$obj == "AllNodes"} {
	foreach obj $Object_List {
	    set old_size $Display_Size($obj)
	    set Display_Size($obj) $Size_Type(${size}_size)
	    set ratio [expr double($Display_Size($obj))/double($old_size)]
	    redraw_node $obj $canvas $ratio $ratio
	}
    } else {
    set old_size $Display_Size($obj)
    set Display_Size($obj) $Size_Type(${size}_size)
    set ratio [expr double($Display_Size($obj))/double($old_size)]
    
    redraw_node $obj $canvas $ratio $ratio
    }
}

proc font_menu {obj win canvas} {
    $win add command -label "Large" -command "changeFont $obj large $canvas"
    $win add command -label "Default" -command "changeFont $obj default $canvas"
    $win add command -label "Small" -command "changeFont $obj small $canvas"
    $win add command -label "Tiny" -command "changeFont $obj tiny $canvas"           
}      

proc object_menu {obj win canvas} {
    $win add command -label "Large" -command "changeSize $obj large $canvas"
    $win add command -label "Default" -command "changeSize $obj default $canvas"
    $win add command -label "Small" -command "changeSize $obj small $canvas"
    $win add command -label "Tiny" -command "changeSize $obj tiny $canvas"           
}      

proc setup_menu_bar {{parent "."} {canvas .c}} {
    global history_menu Font_Type
    
    if {$parent == "."} {
        set win ".mbar"
    } else {
        set win "$parent.mbar"
    }
    
    set fn $Font_Type(default_font)
    
    frame $win -relief raised -bd 2
    pack $win -fill x -side top
    
    menubutton $win.file -text File -underline 0 -menu $win.file.menu -font $fn
    menubutton $win.edit -text Add -underline 0 -menu $win.edit.menu -font $fn
    menubutton $win.font -text "Font Size" -underline 1 -menu $win.font.menu -font $fn
    menubutton $win.object -text "Node Size" -underline 0 -menu $win.object.menu -font $fn
    menubutton $win.help -text Help -underline 0 -menu $win.help.menu -font $fn
    
#    pack [label $win.lbl -text \t\t\t\t] -side left
    pack $win.file $win.edit $win.font $win.object -side left -ipadx 5m
    pack $win.help -side right -ipadx 5m
    
    ####################### File menubutton ########################
    
    menu $win.file.menu -tearoff n

    # $win.file.menu add command -label "New Window" -command "no_op"
    # "new_window"
    # $win.file.menu add command -label "Close Window" -command "no_op"
    # $win.file.menu add separator
    
    $win.file.menu add command -label "Open File ..." \
            -command "load_file $canvas"
    $win.file.menu add command -label "Save File ..." \
            -command "dump_file $canvas"

    $win.file.menu add separator
    $win.file.menu add cascade -label "Print to File" \
        -menu $win.file.menu.print

    $win.file.menu add separator
#    $win.file.menu add command -label "Clear Design" -command "clearGraph $canvas" -font $fn

    $win.file.menu add separator
    
    $win.file.menu add command -label "Exit" -command quit
    
    ######################### Print submenu #######################
    
    menu $win.file.menu.print -tearoff n
    
    $win.file.menu.print add command -label "Portrait ..." -command \
        "print_box $canvas portrait"
    
    $win.file.menu.print add command -label "Landscape ..." -command \
        "print_box $canvas landscape"
        
    ######################### Edit menubutton #######################
    
    menu $win.edit.menu -tearoff n
    
    edit_menu $win.edit.menu $canvas
    
    # $win.edit.menu add separator
    
    #  $win.edit.menu add command -label "Clear Design" -command "clearGraph $parent"
  
    ########################## Font menubutton #######################
    
    menu $win.font.menu -tearoff n
    
    $win.font.menu add cascade -label "All Nodes" -menu \
        $win.font.menu.allNodes

    $win.font.menu add cascade -label "Context Label" -menu \
        $win.font.menu.labelFont

    $win.font.menu add cascade -label "Alternative" -menu \
        $win.font.menu.alternativeFont

    $win.font.menu add separator

    $win.font.menu add cascade -label "Goal" -menu \
        $win.font.menu.goalFont

    $win.font.menu add cascade -label "Task" -menu \
        $win.font.menu.taskFont

    $win.font.menu add cascade -label "Resource" -menu \
        $win.font.menu.resourceFont

    $win.font.menu add cascade -label "Softgoal" -menu \
        $win.font.menu.softgoalFont

    $win.font.menu add separator

    $win.font.menu add cascade -label "Actor" -menu \
        $win.font.menu.actorFont

    $win.font.menu add cascade -label "Role" -menu \
        $win.font.menu.roleFont
        
    $win.font.menu add cascade -label "Position" -menu \
        $win.font.menu.positionFont 
       
    $win.font.menu add cascade -label "Agent" -menu \
        $win.font.menu.agentFont


    ########################## Font size submenu #######################
 
    menu $win.font.menu.allNodes -tearoff n
    font_menu AllNodes $win.font.menu.allNodes $canvas

    menu $win.font.menu.labelFont -tearoff n
    font_menu Label $win.font.menu.labelFont $canvas

    menu $win.font.menu.alternativeFont -tearoff n
    font_menu Alternative $win.font.menu.alternativeFont $canvas

    menu $win.font.menu.actorFont -tearoff n
    font_menu Actor $win.font.menu.actorFont $canvas
    
    menu $win.font.menu.agentFont -tearoff n
    font_menu Agent $win.font.menu.agentFont $canvas
    
    menu $win.font.menu.roleFont -tearoff n
    font_menu Role $win.font.menu.roleFont $canvas
    
    menu $win.font.menu.positionFont -tearoff n
    font_menu Position $win.font.menu.positionFont $canvas
    
    menu $win.font.menu.resourceFont -tearoff n
    font_menu Resource $win.font.menu.resourceFont $canvas
    
    menu $win.font.menu.taskFont -tearoff n
    font_menu Task $win.font.menu.taskFont $canvas
    
    menu $win.font.menu.goalFont -tearoff n
    font_menu Goal $win.font.menu.goalFont $canvas
    
    menu $win.font.menu.softgoalFont -tearoff n
    font_menu Softgoal $win.font.menu.softgoalFont $canvas
    
    ########################## Object menubutton #######################
    
    menu $win.object.menu -tearoff n
    
    $win.object.menu add cascade -label "All Nodes" -menu \
        $win.object.menu.allNodes
    
    $win.object.menu add cascade -label "Context Label" -menu \
        $win.object.menu.labelSize

    $win.object.menu add cascade -label "Alternative" -menu \
        $win.object.menu.alternativeSize

    $win.object.menu add separator

    $win.object.menu add cascade -label "Goal" -menu \
        $win.object.menu.goalSize

    $win.object.menu add cascade -label "Task" -menu \
        $win.object.menu.taskSize

    $win.object.menu add cascade -label "Resource" -menu \
        $win.object.menu.resourceSize

    $win.object.menu add cascade -label "Softgoal" -menu \
        $win.object.menu.softgoalSize

    $win.object.menu add separator

    $win.object.menu add cascade -label "Actor" -menu \
        $win.object.menu.actorSize
 
   $win.object.menu add cascade -label "Role" -menu \
        $win.object.menu.roleSize
        
    $win.object.menu add cascade -label "Position" -menu \
        $win.object.menu.positionSize
 
        
    $win.object.menu add cascade -label "Agent" -menu \
        $win.object.menu.agentSize


    ########################## Object size submenu #######################
 
    menu $win.object.menu.allNodes -tearoff n
    object_menu AllNodes $win.object.menu.allNodes $canvas

    menu $win.object.menu.labelSize -tearoff n
    object_menu Label $win.object.menu.labelSize $canvas

    menu $win.object.menu.alternativeSize -tearoff n
    object_menu Alternative $win.object.menu.alternativeSize $canvas

    menu $win.object.menu.actorSize -tearoff n
    object_menu Actor $win.object.menu.actorSize $canvas
    
    menu $win.object.menu.agentSize -tearoff n
    object_menu Agent $win.object.menu.agentSize $canvas
    
    menu $win.object.menu.roleSize -tearoff n
    object_menu Role $win.object.menu.roleSize $canvas
    
    menu $win.object.menu.positionSize -tearoff n
    object_menu Position $win.object.menu.positionSize $canvas
    
    menu $win.object.menu.resourceSize -tearoff n
    object_menu Resource $win.object.menu.resourceSize $canvas
    
    menu $win.object.menu.taskSize -tearoff n
    object_menu Task $win.object.menu.taskSize $canvas
    
    menu $win.object.menu.goalSize -tearoff n
    object_menu Goal $win.object.menu.goalSize $canvas
    
    menu $win.object.menu.softgoalSize -tearoff n
    object_menu Softgoal $win.object.menu.softgoalSize $canvas
    

    ########################### Help menubutton #######################
    
    menu $win.help.menu -tearoff n
    
    $win.help.menu add command -label "OME Overview" -command\
        "ome_overview"
    
    $win.help.menu add command -label "OME Readme" -command\
        "ome_readme"

    $win.help.menu add command -label "OME Bugs" -command\
        "ome_bugs"
        
    $win.help.menu add command -label "OME Unimplemented" -command\
        "ome_unimplemented"
    
    focus $win
}

##########################################################################
# Create Tool Box Subroutine
##########################################################################

proc design_tool_window {canvas {parent "."} {color wheat}} {
    global Font_Type
    
    set fn1 $Font_Type(default_font)
    set fn2 $Font_Type(default_font)
    
    if {$parent == "."} {
        set tool ".tool"
    } else {
        set tool "$parent.tool"
    }

    frame $tool -relief ridge  
    pack $tool -side top -anchor nw -fill x

#    frame $tool -relief raised
#    pack $tool -side left -anchor nw
    
#    label $tool.title -text "OME Tool Box" -font $fn1
#    pack $tool.title -side top
    
    ##### create frame for each category #####

            
    frame $tool.node1 -relief raise -borderwidth 3
    frame $tool.node1.f1
    frame $tool.node1.f2
  
#    frame $tool.node.f6 
#    label $tool.node.title -text "\nNodes" -font $fn2

    pack  $tool.node1 $tool.node1.f1 $tool.node1.f2 \
	-side left 
#-expand 1 -fill both 
    
    frame $tool.node2 -relief raise -borderwidth 3
    frame $tool.node2.f1
    frame $tool.node2.f2 
    pack $tool.node2 $tool.node2.f1 $tool.node2.f2 \
	-side left 
#-expand 1 -fill both 

    frame $tool.node3 -relief raise -borderwidth 3
    frame $tool.node3.f1 
    pack $tool.node3 $tool.node3.f1 \
	-side left 
#-expand 1 -fill both 

#    pack $tool.node.title $tool.node $tool.node.f1 $tool.node.f2 \
        $tool.node.f3 $tool.node.f4 -side top -expand 1 -fill both

    frame $tool.depend -relief raise -borderwidth 3
    frame $tool.depend.f1
    frame $tool.depend.f2
#    frame $tool.depend.f3 -relief ridge
#    label $tool.depend.title -text "Dependencies" -font $fn2

    pack $tool.depend $tool.depend.f1 $tool.depend.f2 \
        -side left 
#-expand 1 -fill both
    
#    pack $tool.depend.title $tool.depend $tool.depend.f1 $tool.depend.f2 \
        $tool.depend.f3 -side top -expand 1 -fill both

    frame $tool.link -relief raise -borderwidth 3
    frame $tool.link.f1
    frame $tool.link.f2
    frame $tool.link.f3
#    frame $tool.link.f4
#    label $tool.link.title -text "Links" -font $fn2

    pack $tool.link $tool.link.f1 $tool.link.f2 \
        $tool.link.f3 -side left 
#-expand 1 -fill both
    
#    pack $tool.link.title $tool.link $tool.link.f1 $tool.link.f2 \
        $tool.link.f3 -side top -expand 1 -fill both
    
    frame $tool.sep5 -relief raise 
    pack $tool.sep5 -side left -fill both

    frame $tool.relate -relief raise -borderwidth 3
    frame $tool.relate.f1
    frame $tool.relate.f2
    frame $tool.relate.f3
#    label $tool.relate.title -text "Relationships" -font $fn2

    pack $tool.relate $tool.relate.f1 $tool.relate.f2 \
	$tool.relate.f3 -side left 
#-expand 1 -fill both

#    pack $tool.relate.title $tool.relate $tool.relate.f1 \
        $tool.relate.f2 -side top -expand 1 -fill both

    frame $tool.nfr
    pack  $tool.nfr -side left -fill both -expand 1
#    frame $tool.nfr
#    frame $tool.nfr.f1
#    label $tool.nfr.title -text "Others" -font $fn2

#    pack $tool.nfr $tool.nfr.f1 \
        -side right -expand 1 -fill both
    
#    pack $tool.nfr.title $tool.nfr $tool.nfr.f1 \
        -side top -expand 1 -fill both
    
    ##### create node buttons ##### old version
    
#    button $tool.node.f1.label -text Label -font $fn2 \
        -command "new_label $canvas"
#    button $tool.node.f1.alt -text Alternative -font $fn2 \
        -command "new_alternative $canvas"
#    button $tool.node.f2.actor -text Actor -font $fn2 \
        -command "new_actor $canvas"
#    button $tool.node.f2.agent -text Agent -font $fn2 \
        -command "new_agent $canvas"
#    button $tool.node.f2.role -text Role -font $fn2 \
        -command "new_role $canvas"
#    button $tool.node.f3.pos -text Position -font $fn2 \
        -command "new_position $canvas"
#    button $tool.node.f3.res -text Resource -font $fn2 \
        -command "new_resource $canvas"
#    button $tool.node.f4.task -text Task -font $fn2 \
        -command "new_task $canvas"
#    button $tool.node.f4.goal -text Goal -font $fn2 \
        -command "new_goal $canvas"
#    button $tool.node.f4.sgoal -text Softgoal -font $fn2 \
        -command "new_softgoal $canvas"
    
#    pack $tool.node.f1.label $tool.node.f1.alt -side left \
        -expand 1 -fill both
#    pack $tool.node.f2.actor $tool.node.f2.agent $tool.node.f2.role \
        -side left -expand 1 -fill both
#    pack $tool.node.f3.pos $tool.node.f3.res -side left -expand 1 \
        -fill both
#    pack $tool.node.f4.task $tool.node.f4.goal $tool.node.f4.sgoal \
        -side left -expand 1 -fill both

    ### create node buttons new version

    button $tool.node1.f1.task -bitmap @task3.bit -font $fn2 \
        -command "new_task $canvas" -relief ridge
    button $tool.node1.f1.goal -bitmap @goal3.bit -font $fn2 \
        -command "new_goal $canvas" -relief ridge
    button $tool.node1.f2.res  -bitmap @resource3.bit -font $fn2 \
        -command "new_resource $canvas" -relief ridge 
    button $tool.node1.f2.sgoal -bitmap @soft3.bit -font $fn2 \
        -command "new_softgoal $canvas" -relief ridge
       
    button $tool.node2.f1.actor -bitmap @actor3.bit -font $fn2 \
        -command "new_actor $canvas" -relief ridge
    button $tool.node2.f1.role -bitmap @role3.bit -font $fn2 \
        -command "new_role $canvas" -relief ridge
    button $tool.node2.f2.pos -bitmap @position3.bit -font $fn2 \
        -command "new_position $canvas" -relief ridge
    button $tool.node2.f2.agent -bitmap @agent3.bit -font $fn2 \
        -command "new_agent $canvas" -relief ridge

    button $tool.node3.f1.label -bitmap @label3.bit -font $fn2 \
        -command "new_label $canvas" -relief ridge
    button $tool.node3.f1.alt -bitmap @alternative3.bit -font $fn2 \
        -command "new_alternative $canvas" -relief ridge
    


    pack $tool.node1.f1.goal $tool.node1.f1.task   \
	 -side top  -expand 1 -fill both 
# before all these buttons were -expand 1 -fill both   
    pack $tool.node1.f2.res  $tool.node1.f2.sgoal \
	 -side top -expand 1 -fill both 
    pack $tool.node2.f1.actor $tool.node2.f1.role  -side top -fill both 
    pack $tool.node2.f2.pos $tool.node2.f2.agent -side top  -fill both 
    pack $tool.node3.f1.label $tool.node3.f1.alt \
	-side top -fill both 
    
        
    ##### create dependency buttons ##### the old version
    
#    menubutton $tool.depend.f1.res -text Resource -font $fn2 \
        -menu $tool.depend.f1.res.menu -relief ridge
#    menubutton $tool.depend.f1.task -text Task -font $fn2 \
        -menu $tool.depend.f1.task.menu -relief raised
#    menubutton $tool.depend.f2.goal -text Goal -font $fn2 \
        -menu $tool.depend.f2.goal.menu -relief raised
#    menubutton $tool.depend.f2.sgoal -text Softgoal -font $fn2 \
        -menu $tool.depend.f2.sgoal.menu -relief raised
#    menubutton $tool.depend.f3.one_sided -text One-Sided -font $fn2 \
        -menu $tool.depend.f3.one_sided.menu -relief raised
     
#    pack $tool.depend.f1.res $tool.depend.f1.task -side left \
        -expand 1 -fill both -padx 0.5m
#    pack $tool.depend.f2.goal $tool.depend.f2.sgoal -side left \
        -expand 1 -fill both -padx 0.5m
#    pack $tool.depend.f3.one_sided -expand 1 -fill both -padx 0.5m

    ##### create dependency buttons ##### the new version
    
    menubutton $tool.depend.f1.goal -bitmap @goaldep3.bit -font $fn2 \
        -menu $tool.depend.f1.goal.menu -relief ridge
    menubutton $tool.depend.f1.task -bitmap @taskdep3.bit -font $fn2 \
        -menu $tool.depend.f1.task.menu -relief ridge
    menubutton $tool.depend.f2.res -bitmap @resourcedep3.bit -font $fn2 \
        -menu $tool.depend.f2.res.menu -relief ridge
    menubutton $tool.depend.f2.sgoal -bitmap @softgoaldep3.bit -font $fn2 \
        -menu $tool.depend.f2.sgoal.menu -relief ridge
          
    pack $tool.depend.f1.goal $tool.depend.f1.task -side top \
       -fill both -pady 3
    pack $tool.depend.f2.res $tool.depend.f2.sgoal -side top \
        -fill both -pady 3
#    pack $tool.depend.f3.one_sided -expand 1 -fill both -padx 0.5m

        
    ##### resource dependency submenu #####
    
    menu $tool.depend.f2.res.menu -tearoff n
    
    $tool.depend.f2.res.menu add command -label "Committed" -command "new_resource_dependency $canvas"
    $tool.depend.f2.res.menu add command -label "Open" -command "new_resource_dependency $canvas o"
    $tool.depend.f2.res.menu add command -label "Critical" -command "new_resource_dependency $canvas x"
    
    ##### task dependency submenu #####
    
    menu $tool.depend.f1.task.menu -tearoff n
    
    $tool.depend.f1.task.menu add command -label "Committed" -command "new_task_dependency $canvas"
    $tool.depend.f1.task.menu add command -label "Open" -command "new_task_dependency $canvas o"
    $tool.depend.f1.task.menu add command -label "Critical" -command "new_task_dependency $canvas x"
    
    ##### goal dependency submenu #####
    
    menu $tool.depend.f1.goal.menu -tearoff n
    
    $tool.depend.f1.goal.menu add command -label "Committed" -command "new_goal_dependency $canvas"
    $tool.depend.f1.goal.menu add command -label "Open" -command "new_goal_dependency $canvas o"
    $tool.depend.f1.goal.menu add command -label "Critical" -command "new_goal_dependency $canvas x"
    
    ##### softgoal dependency submenu #####
    
    menu $tool.depend.f2.sgoal.menu -tearoff n
    
    $tool.depend.f2.sgoal.menu add command -label "Committed" -command "new_softgoal_dependency $canvas"
    $tool.depend.f2.sgoal.menu add command -label "Open" -command "new_softgoal_dependency $canvas o"
    $tool.depend.f2.sgoal.menu add command -label "Critical" -command "new_softgoal_dependency $canvas x"
        
    ##### create link buttons #####the old version
    
#    button $tool.link.f1.task -text "Task Decompsition" -font $fn2 \
        -command "new_task_decomp_link $canvas"
#    button $tool.link.f2.means -text "Mean Ends" -font $fn2 \
        -command "new_means_ends_link $canvas"
#    menubutton $tool.link.f3.sgoal -text "Softgoal Contribution" -font $fn2 \
        -menu $tool.link.f3.sgoal.menu -relief raised
       
#    pack $tool.link.f1.task -expand 1 -fill both
#    pack $tool.link.f2.means -expand 1 -fill both
#    pack $tool.link.f3.sgoal -expand 1 -fill both

    ##### create link buttons #####the new version by mike
    
    button $tool.link.f1.task -bitmap @taskdec3.bit -text "Task Decompsition" -font $fn2 \
        -command "new_task_decomp_link $canvas" -relief ridge
    button $tool.link.f1.means -bitmap @meanends3.bit -text "Mean Ends" -font $fn2 \
        -command "new_means_ends_link $canvas" -relief ridge
    menubutton $tool.link.f2.sgoal -bitmap @sgoalcon3.bit -text "Softgoal Contribution" -font $fn2 \
        -menu $tool.link.f2.sgoal.menu -relief ridge
    menubutton $tool.link.f2.one_sided -bitmap @onesided3.bit -font $fn2 \
        -menu $tool.link.f2.one_sided.menu -relief ridge
       
    pack $tool.link.f1.task $tool.link.f1.means -side top \
	-fill both
    pack $tool.link.f2.sgoal $tool.link.f2.one_sided -side top\
	-fill both -pady 3


    ##### softgoal contribution submenu #####
    
    menu $tool.link.f2.sgoal.menu -tearoff n
    
    $tool.link.f2.sgoal.menu add command -label "und" -command "new_softgoal_link $canvas"
    $tool.link.f2.sgoal.menu add command -label "pos" -command "new_softgoal_link $canvas +"
    $tool.link.f2.sgoal.menu add command -label "neg" -command "new_softgoal_link $canvas -"
    $tool.link.f2.sgoal.menu add command -label "+sub" -command "new_softgoal_link $canvas +sub"
    $tool.link.f2.sgoal.menu add command -label "-sub" -command "new_softgoal_link $canvas -sub"
    $tool.link.f2.sgoal.menu add command -label "+sup" -command "new_softgoal_link $canvas +sup"
    $tool.link.f2.sgoal.menu add command -label "-sup" -command "new_softgoal_link $canvas -sup"
    $tool.link.f2.sgoal.menu add command -label "and" -command "new_softgoal_link $canvas and"
    $tool.link.f2.sgoal.menu add command -label "or" -command "new_softgoal_link $canvas or"
    $tool.link.f2.sgoal.menu add command -label "eql" -command "new_softgoal_link $canvas eql"
    
    ##### one-sided dependency submenu #####
    
    menu $tool.link.f2.one_sided.menu -tearoff n
    
    $tool.link.f2.one_sided.menu add command -label "Committed" -command "one_sided_dependency $canvas"
    $tool.link.f2.one_sided.menu add command -label "Open" -command "one_sided_dependency $canvas o"
    $tool.link.f2.one_sided.menu add command -label "Critical" -command "one_sided_dependency $canvas x"
    
    ##### create relationship buttons #####the olde version
    
#    button $tool.relate.f1.part -text Part -font $fn2 \
        -command "new_relation $canvas PART"
#    button $tool.relate.f1.plays -text Plays -font $fn2 \
        -command "new_relation $canvas PLAYS"
#    button $tool.relate.f2.covers -text Covers -font $fn2 \
        -command "new_relation $canvas COVERS"
#    button $tool.relate.f2.occupies -text Occupies -font $fn2 \
        -command "new_relation $canvas OCCUPIES"
     
#    pack $tool.relate.f1.part $tool.relate.f1.plays -side left -expand 1 -fill both
#    pack $tool.relate.f2.covers $tool.relate.f2.occupies -side left -expand 1 -fill both

    ##### create relationship buttons #####the new version by mike
    
    button $tool.relate.f1.part -bitmap @part3.bit -text Part -font $fn2 \
        -command "new_relation $canvas PART" -relief ridge
    button $tool.relate.f1.plays -bitmap @plays3.bit -text Plays -font $fn2 \
        -command "new_relation $canvas PLAYS" -relief ridge
    button $tool.relate.f2.covers -bitmap @covers3.bit -text Covers -font $fn2 \
        -command "new_relation $canvas COVERS" -relief ridge
    button $tool.relate.f2.occupies -bitmap @occupies3.bit -text Occupies -font $fn2 \
        -command "new_relation $canvas OCCUPIES" -relief ridge
     
    pack $tool.relate.f1.part $tool.relate.f1.plays -side top -expand 1 -fill both
    pack $tool.relate.f2.covers $tool.relate.f2.occupies -side top -expand 1 -fill both


    ##### create NFR assistant button #####

#    button $tool.nfr.f1.nfra -text "NFR Assistant" -font $fn2 \
        -command "call_NFR $canvas"    
#    pack $tool.nfr.f1.nfra -fill both -expand 1
}

proc design_text_window {canvas {parent "."} {color wheat}} {
    global Text_To_Canvas Canvas_To_Text
    
    if {$parent == "."} {
        set text ".text"
    } else {
        set text "$parent.text"
    }
    
    set Text_To_Canvas($text) $canvas
    set Canvas_To_Text($canvas) $text
    
    frame $text -relief raised
    pack $text -fill x -expand 1
    
    label $text.msg -text "Message: "
    pack $text.msg -side left
    
    label $text.label -text ""
    pack $text.label -anchor w    
}

set Frame_no 0

proc setup_main_canvas {parent {default_font ""}} {
    global Frame_no

    if {$parent == "."} {
        set frame ".frame$Frame_no"
    } else {
        set frame "$parent.f.frame$Frame_no"
    }

    frame $frame
    puts $frame
    set graph_canvas($parent) "$frame.c"
    
    scrollbar $frame.scrolly -command "$frame.c yview"
    scrollbar $frame.scrollx -command "$frame.c xview" -orient horiz

    canvas $frame.c \
            -yscrollcommand "$frame.scrolly set" \
            -xscrollcommand "$frame.scrollx set" \
            -relief raised -width 900 -height 600 \
            -scrollregion {0c 0c 60c 48c}

    pack append $frame $frame.scrolly {right filly} \
        $frame.scrollx {bottom fillx} \
        $frame.c {left fill expand}

    incr Frame_no

    return "$frame $frame.c"
}

set Win_no 0

proc setup_main_window {{label ""}} {
    global Win_no Display_Generic

    incr Win_no
    set new_win ".win$Win_no"
    toplevel $new_win

    if {$label != ""} {
        wm title $new_win "University of Toronto Organization Modelling Environment -- The i* Framework \(User: $label\)"
    } else {
        wm title $new_win "University of Toronto Organizational Design Prototype"
    }

    set background $Display_Generic(background)

    set cf "$new_win.f"
    frame $cf
    
    set bottom "$new_win.bottom"
    frame $bottom
    pack $bottom -fill both -side bottom

    set x [setup_main_canvas $new_win]
    set c [lindex $x 1]
        
    pack [lindex $x 0] -expand 1 -fill both -side bottom
    setup_menu_bar $new_win $c
    pack $cf -expand 1 -fill both
    
    design_tool_window $c $cf $background
    design_text_window $c $bottom $background
    wm geometry $new_win 900x800 
    wm minsize $new_win 900 800
    return [list $new_win $c]
}

proc setup_draw_area {canvas} {
    wm withdraw .

    setup_canvas_background $canvas

    adjust_canvas $canvas
}







